#!/usr/bin/tixwish4.1.8.0
#####################################################################
# Copyright (C) 2000 Artur Trzewik
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
######################################################################
# Header
# CVS - $Id: FormEditor.tcl,v 1.11 2000/10/15 17:19:20 artur Exp $
#
# Information:
# Dies ist eine Startdatei fr Formular Editor
# Es kann die Formulare aus der Beschreibung als XML Dateien (XML DB Schema)
# generieren. Die Formulare knnen dann mit einem Drag&Drop GUI Editor
# angepasst werden. Sie werden als XML Dokumente (DTD dbforms) gespeichert

# Finde Program-Verzeichnis
set sname [info script]
if {$sname==""} {
    # Run interactive for develop purposes
    set progdir /home/artur/diplom/program
} else {
    file lstat $sname stats
    # follow sym links
    if {$stats(type)=="link"} {
	set sname [file readlink $sname]
	if {[file pathtype $sname]=="relative"} {
	    set sname [file join [file dirname [info script]] $sname]
	}
    }
    set progdir [file dirname $sname]
}
wm withdraw .
# Lade Weitere Tcl Komponenten Xotc,tDom,TclX
# und Standardmodule
source [file join $progdir Init.tcl]


#Lade andere Module
source [file join $progdir formgenerator.tcl]
source [file join $progdir formfactory.tcl]
source [file join $progdir guieditor.tcl]
source [file join $progdir editablewidget.tcl]

#Ein FormFactory schaffen fr Benutztung des GUIEditors schaffen
FormFactory mffactory


#Setzte Array lan mit bersetzung-Strings
checklanguage fe
#Dom Schnittstelle auch auf deutsche Umlauten setzten
dom setResultEncoding iso8859-1
# Haupt-Objekt des Programm das die GUI aufbaut und kontroliert
# Nur ein per Ausfhrung

Object FormEditor
FormEditor proc setwin win {
    [self] instvar window
    set window $win
    global lan
    menu $win.mb -tearoff 0
    menu $win.mb.file -tearoff 0
    menu $win.mb.tools -tearoff 0
    menu $win.mb.help -tearoff 0
    
    $win.mb add cascade -label $lan(menu.file.file) -menu $win.mb.file \
	-underline 0
    $win.mb add cascade -label $lan(menu.tools.tools) -menu $win.mb.tools \
	-underline 0
    $win.mb add cascade -label $lan(menu.help.help) -menu $win.mb.help \
	-underline 0
    
    $win.mb.file add command -label $lan(menu.file.readdbschema) \
	-command {FormEditor readDbSchema}
    $win.mb.file add command -label $lan(menu.file.readforms) \
	-command {FormEditor readForms}
    $win.mb.file add command -label $lan(menu.file.saveforms) \
	-command {FormEditor saveForms}
    $win.mb.file add command -label $lan(menu.file.saveformsas) \
	-command {FormEditor saveFormsAs}
    $win.mb.file add separator
    $win.mb.file add command -label $lan(menu.file.exit) \
	-command {FormEditor handle_exit}

    $win.mb.tools add command -label $lan(menu.tools.autoformgen) \
	-command {FormEditor autoFormGen}

    $win.mb.help add command -label $lan(menu.help.help) \
	-command handle_help
    $win.mb.help add command -label $lan(menu.help.about) \
	-command {handle_about FormEditor}
    $win.mb.help add separator
    $win.mb.help add command -label $lan(menu.help.debug) \
	-command {commandloop}
    $win.mb.help add command -label inspector \
	-command {source [file join $progdir inspector.tcl]}
    $win.mb.help add command -label win_inspector \
	-command {source [file join $progdir win_inspector.tcl]}
    $win.mb.help add command -label "view XML Schema" \
	-command {
	    if {![winfo exist .xmlview]} {
		toplevel .xmlview
		tixScrolledText .xmlview.st
		pack  .xmlview.st -expand yes -fill both
	    }
	    .xmlview.st subwidget text delete 1.0 end
	    .xmlview.st subwidget text insert 1.0 [$FormEditor::sdata_node asXML]
	}
    $win.mb.help add command -label "view XML Forms" \
	-command {
	    if {![winfo exist .xmlview]} {
		toplevel .xmlview
		tixScrolledText .xmlview.st
		pack  .xmlview.st -expand yes -fill both
	    }
	    .xmlview.st subwidget text delete 1.0 end
	    .xmlview.st subwidget text insert 1.0 [$FormEditor::data_node asXML]
	}
    
    label $win.lschema -text Schema
    label $win.lforms  -text Forms
    label $win.lschemaf -border 1 -relief solid -bg white
    label $win.lformsf  -border 1 -relief solid -bg white
    button $win.loadschema -image [tix getimage openfile] \
	-command {FormEditor readDbSchema}
    button $win.loadform  -image [tix getimage openfile] \
	-command {FormEditor  readForms}
    grid $win.lschema -column 0 -row 1
    grid $win.lforms -column 0 -row 0
    grid $win.lschemaf -column 1 -row 1 -sticky ew
    grid $win.lformsf -column 1 -row 0 -sticky ew
    grid $win.loadschema -column 2 -row 1 -sticky w
    grid $win.loadform -column 2 -row 0 -sticky w
    tixScrolledTList $win.list  -options {
	tlist.orient horizontal
	tlist.selectMode single
	tlist.command {FormEditor executeForm}
	tlist.width 60
	tlist.height 10
    }
    grid $win.list -sticky nsew -column 0 -row 2 -columnspan 2

    frame $win.box  -relief raised -bd 2
    button $win.box.newform -text $lan(button.newform) \
	-command {FormEditor newForm}
    button $win.box.editform -text $lan(button.editform) \
	-command {FormEditor editForm}
    button $win.box.deleteform -text $lan(button.deleteform) \
	-command {FormEditor deleteForm}
    pack $win.box.newform $win.box.editform $win.box.deleteform \
	-side top -anchor n -fill x -padx 4 -pady 2
    grid $win.box -colum 2 -row 2 -sticky nsew

    grid columnconfigure . 1 -weight 1
    grid rowconfigure . 2 -weight 1
    
    if {$win==""} {
	. configure -menu $win.mb
	wm protocol . WM_DELETE_WINDOW "[self] handle_exit"
    } else {
	$win configure -menu $win.mb
	wm protocol $win WM_DELETE_WINDOW "[self] handle_exit"
    }
}
FormEditor proc readForms {{file {}} {findschema 1}} {
    [self] instvar window data_node dom_doc db_name file_name
    # !!! Suche andere Methode das Tree window zu finden
    set types {
	{"XML Forms"		{.xmlforms}}
	{"All files"		*}
    }
    if {$file==""} {
	set file [tk_getOpenFile -filetypes $types -parent .]
	if {$file == ""} { return }
    }
    set fileh [open $file]
    if {[info exist dom_doc]} {
	set alt_dom_doc $dom_doc
    } else {
	set alt_dom_doc {}
    }
    set dom_doc [dom parse [read $fileh]]
    close $fileh
    set data_node [$dom_doc documentElement]
    if {[$data_node nodeName]!="dbforms"} {
	global lan
	$dom_doc delete
	set dom_doc ""
	tk_messageBox -message [format $lan(message.wrongxml) xmlforms] -type ok -icon error
	return
    }
    set db_name [$data_node getAttribute name]
    set file_name $file
    [self] showForms
    $window.lformsf configure -text [file tail $file]
    if {$findschema} {
	[self] findSchema
    }
    if {$alt_dom_doc!=""} {
	$alt_dom_doc delete
    }
}
FormEditor proc showForms {} {
    [self] instvar data_node window form_nodelist
    set flist [$window.list subwidget tlist]
    $flist delete 0 end
    set form_nodelist {}
    foreach node [$data_node childNodes] {
	if {[$node nodeType]=="ELEMENT_NODE" && [$node nodeName]=="form"} then {
	    [self] addForm $node
	} 
    }
    
}
FormEditor proc findSchema {} {
    [self] instvar window sdata_node sdom_doc data_node schema_file db_name file_name
    if {![info exist sdata_node]} {
	set schema_file {}
	# Suche die ProcessingInstuction dbschema
	foreach node [$data_node selectNodes child::processing-instruction()] {
	    if {[$node target]=="dbschema"} {
		set schema_file [$node data]
		break
	    }
	}
	if {$schema_file!=""} {
	    if {![file exist $schema_file]} {
		# Probiere da wo auch form Datei war
		set schema_file [file join [file dirname $file_name] [file tail $schema_file]]
	    }
	} else {
	    # Leite den Namen von dbforms Attribut name
	    set schema_file [file join [file dirname $file_name] $db_name.xmldbschema]
	}
	#puts "file $schema_file"
	if {[file exist $schema_file] && [file isfile $schema_file]} {
	    global lan
	    if {[tk_messageBox -message [format $lan(message.readschema) $schema_file] -type yesno -icon question]=="yes"} {
		[self] readDbSchema $schema_file
	    }
	    return
	}
    }
}
FormEditor proc readDbSchema {{file {}}} {
    [self] instvar sdata_node sdom_doc db_name window schema_file 
    # !!! Suche andere Methode das Tree window zu finden
    set types {
	{"XML DB Schema"	{.xmldbschema}}
	{"All files"		*}
    }
    if {$file==""} {
	set file [tk_getOpenFile -filetypes $types -parent .]
	if {$file == ""} { return }
    }
    if {[info exist sdom_doc]} {
	set alt_dom_doc $sdom_doc
    } else {
	set alt_dom_doc {}
    }
    set flist [$window.list subwidget tlist]
    set fileh [open $file]
    set xmlfile [read $fileh]
    close $fileh
    set sdom_doc [dom parse $xmlfile]
    set sdata_node [$sdom_doc documentElement]
    if {[$sdata_node nodeName]!="database"} {
	global lan
	$sdom_doc delete
	set sdom_doc ""
	tk_messageBox -message [format $lan(message.wrongxml) xmldbschema] -type ok -icon error
	return
    }
    $window.lschemaf configure -text [file tail $file]
    set schema_file $file
    set db_name [$sdata_node getAttribute name]
    if {$alt_dom_doc!=""} {
	$alt_dom_doc delete
    }
}
FormEditor proc saveForms {} {
    [self] instvar file_name
    if {![info exists file_name]} {
	[self] saveFormsAs
    } else {
	[self] saveFormsAs $file_name
    }
}
FormEditor proc saveFormsAs {{f_name {}}} {
   [self] instvar data_node file_name window db_name
    set types {
        {"XML files"         {.xmlforms}  }
        {"All files"            *}
    }
    if {$f_name!=""} {
	set file_name $f_name
    } else {
	if {![info exists file_name]} {
	    set file_name $db_name.xmlforms
	}
	set f_name [tk_getSaveFile -filetypes $types -parent . \
            -initialfile $file_name -defaultextension .xmlforms]
	if {$f_name==""} return
    }
    set file_name $f_name
    set fh [open $file_name w]
    puts $fh "<?xml version='1.0' encoding='iso-8859-1'?>\n<!DOCTYPE dbforms SYSTEM \"xmlforms.dtd\">\n[$data_node asXML]\n"
    close $fh
    $window.lformsf configure -text [file tail $file_name]
}
FormEditor proc addForm {form_node} {
    [self] instvar window form_nodelist
    set flist [$window.list subwidget tlist]
    $flist insert end -itemtype imagetext \
	-text [$form_node getAttribute name] \
	-image [tix getimage openfold]
    lappend form_nodelist $form_node
}
FormEditor proc applyChanges {form_node} {
    [self] instvar window form_nodelist
    set flist [$window.list subwidget tlist]
    set id [lsearch $form_nodelist $form_node]
    if {$id<0} return
    $flist entryconfigure $id -text [$form_node getAttribute name]
}
FormEditor proc newForm {} {
    [self] instvar sdata_node data_node dom_doc db_name sdom_doc schema_question
    global lan
    # Gibt Warnungen, wenn wahrschienliche schlecht Benutzung des Programs
    if {![info exist dom_doc]} {
	if {![info exist sdata_node]} {
	    if {[tk_messageBox -title $lan(dialog.warning) \
		    -message $lan(dialog.formwarning) -icon question -type okcancel]!="ok"} { return }
	    set schema_question 1
	} else {
	    if {[tk_messageBox -title $lan(dialog.warning) \
		     -message $lan(dialog.noformwarning) -icon question -type okcancel]!="ok"} { return }
	}
    }
    set prop  {
	{{type OLabelEntry} {name name} {label name} {textwidth 20}}
	{{type OLabelEntry} {name primarykey} {label primarykey} {textwidth 20}}
    }
    # Ist DB_Schema Vorhanden
    if {![info exist sdata_node]} {
	set tprop {{type OLabelEntry} {name table} {label table} {textwidth 20}}    } else {
	    set tprop {{type OComboBoxN} {name table} {label table} {notifyproc {FormEditor notify_table}} }
	    # tDom Bug, workaround
	    $sdom_doc documentElement
	    keylset tprop valuelist [$sdata_node selectNodes //descendant::table/attribute::name]
	    lappend prop {{type OCheckButton} {name useass} {label {use Assistent}} {default 0}}
    }
    OInputDialog [self]::newformdialog [linsert $prop 1 $tprop] {}
    if {[set ret [[self]::newformdialog execute]]!=""} {
	set rkeys [keylkeys ret]
	if {![lcontain $rkeys name] || ![lcontain $rkeys table] || !([lcontain $rkeys primarykey] || [lcontain $rkeys useass])} {
	    tk_messageBox -title $lan(dialog.error) \
		    -message $lan(dialog.notallinputs) -icon error -type ok
	    return
	}
	if {![info exist dom_doc]} {
	    set dom_doc [dom createDocument dbforms]
	    set data_node [$dom_doc documentElement]
	    set db_name manuforms
	    $data_node setAttribute name $db_name
	}
	if {[$data_node selectNodes child::form\[@name=\"[keylget ret name]\"\]]!=""} {
	    tk_messageBox -title $lan(dialog.error) \
		    -message $lan(dialog.namealreadyexist) -icon warning -type ok
	    return
	}
	if {[lcontain $rkeys useass] && [keylget ret useass]} {
	    # Erzeuge durch Assistenten
	    set form_node [$dom_doc createElement form]
	    $form_node setAttribute name [keylget ret name]
	    $data_node appendChild $form_node
	    set table_node [$sdata_node selectNodes descendant::table\[@name=\"[keylget ret table]\"\]]
	    #puts "TNODE $table_node [keylget ret table]"
	    FormGenerator autogenForm $form_node $table_node 
	} else {
	    set form_node [$dom_doc createElement form]
	    $form_node setAttribute name [keylget ret name]
	    $form_node setAttribute table [keylget ret table]
	    $form_node setAttribute primarykey [keylget ret primarykey]
	    $data_node appendChild $form_node
	    set frame_node [$dom_doc createElement frame]
	    $frame_node setAttribute manager pack
	    $form_node appendChild $frame_node
	}
	[self] addForm $form_node
    }
    [self]::newformdialog destroy
}
# Ein Notify_hndler fr NewForm Dialog
# Setzt automatisch primary keys passend zu Tabelle
FormEditor proc notify_table {table} {
    [self] instvar sdata_node
    set table_node [$sdata_node selectNodes descendant::table\[@name=\"$table\"\]]
    set primattr [tdom_getprimattr $table_node]
    # Wenn prim Schlssel nicht vorhanden dann alle Attribtute
    if {$primattr==""} {
	set primattr [$table_node selectNodes {child::node()[not(self::table)]/descendant-or-self::attr/attribute::name}]
    }
    
    [self]::newformdialog::inputframe::primarykey setvalue $primattr
}
FormEditor proc deleteForm {} {
    [self] instvar window form_nodelist
    set flist [$window.list subwidget tlist]
    set id [lindex [$flist info selection] 0]
    if {$id!=""} {
	$flist delete $id
	set form_node [lindex $form_nodelist $id]
	[$form_node parentNode] removeChild $form_node
	set form_nodelist [lreplace $form_nodelist $id $id]
    }
}
FormEditor proc editForm {} {
    [self] instvar window form_nodelist
    set flist [$window.list subwidget tlist]
    set id [lindex [$flist info selection] 0]
    #puts "ID $id"
    if {$id!=""} {[self] executeForm $id}
}
FormEditor proc autoFormGen {} {
    [self] instvar data_node dom_doc sdom_doc sdata_node db_name schema_file
    if {![info exist sdata_node]} {
	global lan
	#$dom_doc delete
	tk_messageBox -message $lan(message.noxmlschema_gen) -type ok -icon error
	return
    }
    # tDom Bug, workaround
    $sdom_doc documentElement
    if {![info exist dom_doc]} {
	#puts "dom creator"
	#Es existiertieren keine Formulare
	set dom_doc [dom createDocument dbforms]
	set data_node [$dom_doc documentElement]
	$data_node setAttribute name $db_name
	#Processing Command aus schema zu form transportieren
	#z.B <? dbconnect .....?>
	foreach node [$sdata_node selectNodes child::processing-instruction()] {
	    $data_node appendChild [$dom_doc createProcessingInstruction \
					[$node target] [$node data]]
	}
	# dbschema Datei Lokation als ProcessingInstruction anbinden
	$data_node appendChild [$dom_doc createProcessingInstruction dbschema $schema_file]
    }

    FormGenerator autogenAll $sdata_node $data_node
    [self] showForms
}
FormEditor proc executeForm num {
    [self] instvar window data_node sdata_node sdom_doc form_nodelist schema_question
    set flist [$window.list subwidget tlist]
    set tablename  [[lindex $form_nodelist $num] getAttribute table]
    set form_node  [lindex $form_nodelist $num]
    set schema_node {}
    if {![info exist sdata_node]} {
	if {![info exist schema_question]} {
	    global lan
	    if {[tk_messageBox -message $lan(message.noxmlschema) -type yesno -icon question]!="yes"} {
		return
	    }
	    # die Frage nicht nochmal stellen? Es weit was er tut.
	    set schema_question 1
	}
    } else {
	if {$sdata_node!=""} {
	    set schema_node [$sdata_node selectNodes descendant::table\[@name=\"$tablename\"\]]
	} {
	    set schema_node {}
	}
	if {$schema_node==""} {
	    global lan
	    if {[tk_messageBox -message $lan(message.noschema) -type yesno -icon question]!="yes"} {
		return
	    }
	}
    }
    # berprfe ob bereit editiert
    foreach editor [DBGuiEditor info instances] {
	if {$form_node==[$editor set form_node]} {
	    #puts "Bereits Editiert"
	    return
	}
    }
    DBGuiEditor [Object autoname myDBGuiEditor] -form_node $form_node \
	-schema_node $schema_node
}
FormEditor proc handle_exit {} {
    global lan
    if {[tk_messageBox -title $lan(dialog.endtitle) \
	     -message $lan(dialog.endmsg) -icon question -type okcancel]=="ok"} { exit } 
}
FormEditor setwin {}
#FormEditor readDbSchema mydb.xmldbschema
#FormEditor readForms mydb.xmlforms
#FormEditor executeForm 0
if {!$tcl_interactive && $argc>=1} {
    if {$argc==2} {
	FormEditor readDbSchema [lindex $argv 0]
	FormEditor readForms [lindex $argv 1] 0
    } else {
	FormEditor readForms [lindex $argv 0]
    }
}
wm deiconify .
